# Alyra Final Project's Requirements


## User Interface 

Your project must:

- [x] have a User Interface
- [x] be a Truffle projet
- [x] use Truffle's `react` box
- [x] contain a README that:
    - [x] introduces the projet and its concept(s)
    - [x] details its features
    - [x] describe how to install and launch it
- have the following features: 
    - [x] display the current Ethereum account
    - [ ] sign transactions 
    - [x] interact with your contract's state
    - [x] display your contract's state
- [x] be deployed on *Heroku* and you must be able to open a URL to interact with the application
- [x] You must provide us with a link to your **Github** repository that must be **public**. 


## Smart Contracts

Your smart-contracts must:

- [x] use the latest compiler version
- [x] contains comments in accordance to [Solidity's official documentation](https://solidity.readthedocs.io/en/v0.4.21/layout-of-source-files.html#comments)
- [x] generate a **README** per smart-contract using  **NatSpec**
- [x] be **secured** and **optimized**

Your project must:
- [x] contain a file named `avoiding_common_attacks.md` explaining the measures you took to prevent the **security breaches**
- [x] contain a file named `deployed_addresses.md` with the differents **addresses of your smart-contrats** on the aforementioned testnet
- [x] Your application must be **deployed** on an **Ethereum testnet** (FYI: the smart-contracts must be deployed with Truffle) 
- [x] use the packages `dotenv` and `@truffle/hdwallet-provider`


## Tests

You project must:

- [x] contain integration tests for each smart-contract
- [x] contain a file named `tests_explication.md` explaining  **why you wrote these thests**


## ⭐️ Bonus

- [x] A file named `design_pattern_decisions.md` if present to list which **design patterns** you chose and why.
This [link](https://fravoll.github.io/solidity-patterns/) may help. 


## Example (Git Repository)

You can find [here](https://github.com/Alyra-school/projet-final) an example of the expected directory tree including the expected items. 
You can then clone it to bootstrap your project.

